@@ -1,3 +1,7 @@
+(* TEST
+   * expect
+*)
+
 type (_, _) t =
   | Nil : ('tl, 'tl) t
   | Cons : 'a * ('b, 'tl) t -> ('a * 'b, 'tl) t;;
@@ -8,6 +12,8 @@ type (_, _) t =
     Nil : ('tl, 'tl) t
   | Cons : 'a * ('b, 'tl) t -> ('a * 'b, 'tl) t
 Line _, characters 9-43:
+  let get1 (Cons (x, _) : (_ * 'a, 'a) t) = x ;; (* warn, cf PR#6993 *)
+           ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 Warning 8: this pattern-matching is not exhaustive.
 Here is an example of a case that is not matched:
 Nil
@@ -21,6 +27,8 @@ let get1' = function
 val get1' : ('b * 'a as 'a, 'a) t -> 'b = <fun>
 |}, Principal{|
 Line _, characters 4-7:
+    | Nil -> assert false ;; (* ok *)
+      ^^^
 Error: This pattern matches values of type ('b * 'a, 'b * 'a) t
        but a pattern was expected which matches values of type
          ('b * 'a, 'a) t
